@@ -40,6 +40,12 @@ main (int argc, char *argv[])
 	printf("Cannot bind to %s\n", argv[1]);
 	return 1;
     }
+
+    printf("corba = %d, empty = %d, foobar = %d\n",
+	CORBA_Object_is_a(empty_client, "IDL:CORBA/Object:1.0", &ev),
+	CORBA_Object_is_a(empty_client, "IDL:Empty:1.0", &ev),
+	CORBA_Object_is_a(empty_client, "IDL:Foo/Bar:1.0", &ev));
+
     for(i = 0; i < niters; i++) {
 	    Empty_doNothing(empty_client, &ev);
 	    if(ev._major != CORBA_NO_EXCEPTION) {
